"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const Authorization_1 = __importDefault(require("./Authorization"));
const Permission_1 = __importDefault(require("./Permission"));
const WarrantClient_1 = __importDefault(require("../WarrantClient"));
const ObjectType_1 = require("../types/ObjectType");
class Role {
    constructor(roleId, name, description) {
        this.roleId = roleId;
        this.name = name;
        this.description = description;
    }
    //
    // Static methods
    //
    static create(role) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield WarrantClient_1.default.httpClient.post({
                    url: "/v1/roles",
                    data: role,
                });
                return new Role(response.roleId, response.name, response.description);
            }
            catch (e) {
                throw e;
            }
        });
    }
    static get(roleId) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield WarrantClient_1.default.httpClient.get({
                    url: `/v1/roles/${roleId}`,
                });
                return new Role(response.roleId, response.name, response.description);
            }
            catch (e) {
                throw e;
            }
        });
    }
    static update(roleId, role) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield WarrantClient_1.default.httpClient.put({
                    url: `/v1/roles/${roleId}`,
                    data: role,
                });
                return new Role(response.roleId, response.name, response.description);
            }
            catch (e) {
                throw e;
            }
        });
    }
    static delete(roleId) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield WarrantClient_1.default.httpClient.delete({
                    url: `/v1/roles/${roleId}`,
                });
            }
            catch (e) {
                throw e;
            }
        });
    }
    static listRoles(listOptions = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield WarrantClient_1.default.httpClient.get({
                    url: "/v1/roles",
                    params: listOptions,
                });
                return response.map((role) => new Role(role.roleId, role.name, role.description));
            }
            catch (e) {
                throw e;
            }
        });
    }
    static listRolesForUser(userId, listOptions = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield WarrantClient_1.default.httpClient.get({
                    url: `/v1/users/${userId}/roles`,
                    params: listOptions,
                });
                return response.map((role) => new Role(role.roleId, role.name, role.description));
            }
            catch (e) {
                throw e;
            }
        });
    }
    static assignRoleToUser(userId, roleId) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield WarrantClient_1.default.httpClient.post({
                    url: `/v1/users/${userId}/roles/${roleId}`,
                });
                return new Role(response.roleId, response.name, response.description);
            }
            catch (e) {
                throw e;
            }
        });
    }
    static removeRoleFromUser(userId, roleId) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield WarrantClient_1.default.httpClient.delete({
                    url: `/v1/users/${userId}/roles/${roleId}`,
                });
            }
            catch (e) {
                throw e;
            }
        });
    }
    // Instance methods
    listPermissions(listOptions = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return Permission_1.default.listPermissionsForRole(this.roleId, listOptions);
        });
    }
    assignPermission(permissionId) {
        return __awaiter(this, void 0, void 0, function* () {
            return Permission_1.default.assignPermissionToRole(this.roleId, permissionId);
        });
    }
    removePermission(permissionId) {
        return __awaiter(this, void 0, void 0, function* () {
            return Permission_1.default.removePermissionFromRole(this.roleId, permissionId);
        });
    }
    hasPermission(permissionId, context = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return Authorization_1.default.hasPermission({ permissionId: permissionId, subject: { objectType: ObjectType_1.ObjectType.Role, objectId: this.roleId }, context: context });
        });
    }
    // WarrantObject methods
    getObjectType() {
        return ObjectType_1.ObjectType.Role;
    }
    getObjectId() {
        return this.roleId;
    }
}
exports.default = Role;
