"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const Authorization_1 = __importDefault(require("./Authorization"));
const Feature_1 = __importDefault(require("./Feature"));
const Permission_1 = __importDefault(require("./Permission"));
const PricingTier_1 = __importDefault(require("./PricingTier"));
const Role_1 = __importDefault(require("./Role"));
const Tenant_1 = __importDefault(require("./Tenant"));
const WarrantClient_1 = __importDefault(require("../WarrantClient"));
const ObjectType_1 = require("../types/ObjectType");
const WarrantModule_1 = __importDefault(require("./WarrantModule"));
class User {
    constructor(userId, email) {
        this.userId = userId;
        this.email = email;
    }
    //
    // Static methods
    //
    static create(user = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield WarrantClient_1.default.httpClient.post({
                    url: "/v1/users",
                    data: user,
                });
                return new User(response.userId, response.email);
            }
            catch (e) {
                throw e;
            }
        });
    }
    static batchCreate(users) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield WarrantClient_1.default.httpClient.post({
                    url: "/v1/users",
                    data: users,
                });
                return response.map((user) => new User(user.userId, user.email));
            }
            catch (e) {
                throw e;
            }
        });
    }
    static get(userId) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield WarrantClient_1.default.httpClient.get({
                    url: `/v1/users/${userId}`,
                });
                return new User(response.userId, response.email);
            }
            catch (e) {
                throw e;
            }
        });
    }
    static update(userId, user) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield WarrantClient_1.default.httpClient.put({
                    url: `/v1/users/${userId}`,
                    data: user,
                });
                return new User(response.userId, response.email);
            }
            catch (e) {
                throw e;
            }
        });
    }
    static delete(userId) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield WarrantClient_1.default.httpClient.delete({
                    url: `/v1/users/${userId}`,
                });
            }
            catch (e) {
                throw e;
            }
        });
    }
    static listUsers(listOptions = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield WarrantClient_1.default.httpClient.get({
                    url: "/v1/users",
                    params: listOptions,
                });
                return response.map((user) => new User(user.userId, user.email));
            }
            catch (e) {
                throw e;
            }
        });
    }
    static listUsersForTenant(tenantId, listOptions = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield WarrantClient_1.default.httpClient.get({
                    url: `/v1/tenants/${tenantId}/users`,
                    params: listOptions,
                });
                return response.map((user) => new User(user.userId, user.email));
            }
            catch (e) {
                throw e;
            }
        });
    }
    static assignUserToTenant(tenantId, userId, role) {
        return __awaiter(this, void 0, void 0, function* () {
            return WarrantModule_1.default.create({
                object: {
                    objectType: "tenant",
                    objectId: tenantId,
                },
                relation: role,
                subject: {
                    objectType: "user",
                    objectId: userId,
                }
            });
        });
    }
    static removeUserFromTenant(tenantId, userId, role) {
        return __awaiter(this, void 0, void 0, function* () {
            return WarrantModule_1.default.delete({
                object: {
                    objectType: "tenant",
                    objectId: tenantId,
                },
                relation: role,
                subject: {
                    objectType: "user",
                    objectId: userId,
                }
            });
        });
    }
    //
    // Instance methods
    //
    listTenants(listOptions = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return Tenant_1.default.listTenantsForUser(this.userId, listOptions);
        });
    }
    listRoles(listOptions = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return Role_1.default.listRolesForUser(this.userId, listOptions);
        });
    }
    assignRole(roleId) {
        return __awaiter(this, void 0, void 0, function* () {
            return Role_1.default.assignRoleToUser(this.userId, roleId);
        });
    }
    removeRole(roleId) {
        return __awaiter(this, void 0, void 0, function* () {
            return Role_1.default.removeRoleFromUser(this.userId, roleId);
        });
    }
    listPermissions(listOptions = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return Permission_1.default.listPermissionsForUser(this.userId, listOptions);
        });
    }
    assignPermission(permissionId) {
        return __awaiter(this, void 0, void 0, function* () {
            return Permission_1.default.assignPermissionToUser(this.userId, permissionId);
        });
    }
    removePermission(permissionId) {
        return __awaiter(this, void 0, void 0, function* () {
            return Permission_1.default.removePermissionFromUser(this.userId, permissionId);
        });
    }
    hasPermission(permissionId, context = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return Authorization_1.default.hasPermission({ permissionId: permissionId, subject: { objectType: ObjectType_1.ObjectType.User, objectId: this.userId }, context: context });
        });
    }
    listPricingTiers(listOptions = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return PricingTier_1.default.listPricingTiersForUser(this.userId, listOptions);
        });
    }
    assignPricingTier(pricingTierId) {
        return __awaiter(this, void 0, void 0, function* () {
            return PricingTier_1.default.assignPricingTierToUser(this.userId, pricingTierId);
        });
    }
    removePricingTier(pricingTierId) {
        return __awaiter(this, void 0, void 0, function* () {
            return PricingTier_1.default.removePricingTierFromUser(this.userId, pricingTierId);
        });
    }
    listFeatures(listOptions = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return Feature_1.default.listFeaturesForUser(this.userId, listOptions);
        });
    }
    assignFeature(featureId) {
        return __awaiter(this, void 0, void 0, function* () {
            return Feature_1.default.assignFeatureToUser(this.userId, featureId);
        });
    }
    removeFeature(featureId) {
        return __awaiter(this, void 0, void 0, function* () {
            return Feature_1.default.removeFeatureFromUser(this.userId, featureId);
        });
    }
    hasFeature(featureId, context = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return Authorization_1.default.hasFeature({ featureId: featureId, subject: { objectType: ObjectType_1.ObjectType.User, objectId: this.userId }, context: context });
        });
    }
    // WarrantObject methods
    getObjectType() {
        return ObjectType_1.ObjectType.User;
    }
    getObjectId() {
        return this.userId;
    }
}
exports.default = User;
